@@ -1,8 +1,8 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 5                                                        |
+  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2014 The PHP Group                                |
+  | Copyright (c) 1997-2016 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -35,7 +35,7 @@
 /* Cache of the server supported datatypes, initialized in handle_factory */
 zval* pdo_dblib_datatypes;
 
-static int dblib_fetch_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, zval *info TSRMLS_DC)
+static int dblib_fetch_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, zval *info)
 {
 	pdo_dblib_db_handle *H = (pdo_dblib_db_handle *)dbh->driver_data;
 	pdo_dblib_err *einfo = &H->err;
@@ -61,18 +61,19 @@ static int dblib_fetch_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, zval *info TSRMLS
 		msg, einfo->dberr, einfo->severity, stmt ? stmt->active_query_string : "");
 
 	add_next_index_long(info, einfo->dberr);
-	add_next_index_string(info, message, 0);
+	add_next_index_string(info, message);
+	efree(message);
 	add_next_index_long(info, einfo->oserr);
 	add_next_index_long(info, einfo->severity);
 	if (einfo->oserrstr) {
-		add_next_index_string(info, einfo->oserrstr, 1);
+		add_next_index_string(info, einfo->oserrstr);
 	}
 
 	return 1;
 }
 
 
-static int dblib_handle_closer(pdo_dbh_t *dbh TSRMLS_DC)
+static int dblib_handle_closer(pdo_dbh_t *dbh)
 {
 	pdo_dblib_db_handle *H = (pdo_dblib_db_handle *)dbh->driver_data;
 
@@ -91,11 +92,11 @@ static int dblib_handle_closer(pdo_dbh_t *dbh TSRMLS_DC)
 	return 0;
 }
 
-static int dblib_handle_preparer(pdo_dbh_t *dbh, const char *sql, long sql_len, pdo_stmt_t *stmt, zval *driver_options TSRMLS_DC)
+static int dblib_handle_preparer(pdo_dbh_t *dbh, const char *sql, size_t sql_len, pdo_stmt_t *stmt, zval *driver_options)
 {
 	pdo_dblib_db_handle *H = (pdo_dblib_db_handle *)dbh->driver_data;
 	pdo_dblib_stmt *S = ecalloc(1, sizeof(*S));
-	
+
 	S->H = H;
 	stmt->driver_data = S;
 	stmt->methods = &dblib_stmt_methods;
@@ -105,7 +106,7 @@ static int dblib_handle_preparer(pdo_dbh_t *dbh, const char *sql, long sql_len,
 	return 1;
 }
 
-static long dblib_handle_doer(pdo_dbh_t *dbh, const char *sql, long sql_len TSRMLS_DC)
+static zend_long dblib_handle_doer(pdo_dbh_t *dbh, const char *sql, size_t sql_len)
 {
 	pdo_dblib_db_handle *H = (pdo_dblib_db_handle *)dbh->driver_data;
 	RETCODE ret, resret;
@@ -119,7 +120,7 @@ static long dblib_handle_doer(pdo_dbh_t *dbh, const char *sql, long sql_len TSRM
 	if (FAIL == dbsqlexec(H->link)) {
 		return -1;
 	}
-	
+
 	resret = dbresults(H->link);
 
 	if (resret == FAIL) {
@@ -141,84 +142,113 @@ static long dblib_handle_doer(pdo_dbh_t *dbh, const char *sql, long sql_len TSRM
 	return DBCOUNT(H->link);
 }
 
-static int dblib_handle_quoter(pdo_dbh_t *dbh, const char *unquoted, int unquotedlen, char **quoted, int *quotedlen, enum pdo_param_type paramtype TSRMLS_DC)
+static int dblib_handle_quoter(pdo_dbh_t *dbh, const char *unquoted, size_t unquotedlen, char **quoted, size_t *quotedlen, enum pdo_param_type paramtype)
 {
-	pdo_dblib_db_handle *H = (pdo_dblib_db_handle *)dbh->driver_data;
-	char *q;
-	int l = 1;
 
-	*quoted = q = safe_emalloc(2, unquotedlen, 3);
-	*q++ = '\'';
+	int useBinaryEncoding = 0;
+	const char * hex = "0123456789abcdef";
+	int i;
+	char * q;
+	*quotedlen = 0;
 
-	while (unquotedlen--) {
-		if (*unquoted == '\'') {
-			*q++ = '\'';
-			*q++ = '\'';
-			l += 2;
-		} else {
-			*q++ = *unquoted;
-			++l;
+	/*
+	 * Detect quoted length and if we should use binary encoding
+	 */
+	for(i=0;i<unquotedlen;i++) {
+		if( 32 > unquoted[i] || 127 < unquoted[i] ) {
+			useBinaryEncoding = 1;
+			break;
 		}
-		unquoted++;
+		if(unquoted[i] == '\'') ++*quotedlen;
+		++*quotedlen;
 	}
 
-	*q++ = '\'';
-	*q++ = '\0';
-	*quotedlen = l+1;
-	
+	if(useBinaryEncoding) {
+		/*
+		 * Binary safe quoting
+		 * Will implicitly convert for all data types except Text, DateTime & SmallDateTime
+		 *
+		 */
+		*quotedlen = (unquotedlen * 2) + 2; /* 2 chars per byte +2 for "0x" prefix */
+		q = *quoted = emalloc(*quotedlen);
+
+		*q++ = '0';
+		*q++ = 'x';
+		for (i=0;i<unquotedlen;i++) {
+			*q++ = hex[ (*unquoted>>4)&0xF];
+			*q++ = hex[ (*unquoted++)&0xF];
+		}
+	} else {
+		/* Alpha/Numeric Quoting */
+		*quotedlen += 2; /* +2 for opening, closing quotes */
+		q  = *quoted = emalloc(*quotedlen);
+		*q++ = '\'';
+
+		for (i=0;i<unquotedlen;i++) {
+			if (unquoted[i] == '\'') {
+				*q++ = '\'';
+				*q++ = '\'';
+			} else {
+				*q++ = unquoted[i];
+			}
+		}
+		*q++ = '\'';
+	}
+
+	*q = 0;
+
 	return 1;
 }
 
-static int pdo_dblib_transaction_cmd(const char *cmd, pdo_dbh_t *dbh TSRMLS_DC)
+static int pdo_dblib_transaction_cmd(const char *cmd, pdo_dbh_t *dbh)
 {
 	pdo_dblib_db_handle *H = (pdo_dblib_db_handle *)dbh->driver_data;
-	RETCODE ret;
-	
+
 	if (FAIL == dbcmd(H->link, cmd)) {
 		return 0;
 	}
-	
+
 	if (FAIL == dbsqlexec(H->link)) {
 		return 0;
 	}
-	
+
 	return 1;
 }
 
-static int dblib_handle_begin(pdo_dbh_t *dbh TSRMLS_DC)
+static int dblib_handle_begin(pdo_dbh_t *dbh)
 {
-	return pdo_dblib_transaction_cmd("BEGIN TRANSACTION", dbh TSRMLS_CC);
+	return pdo_dblib_transaction_cmd("BEGIN TRANSACTION", dbh);
 }
 
-static int dblib_handle_commit(pdo_dbh_t *dbh TSRMLS_DC)
+static int dblib_handle_commit(pdo_dbh_t *dbh)
 {
-	return pdo_dblib_transaction_cmd("COMMIT TRANSACTION", dbh TSRMLS_CC);
+	return pdo_dblib_transaction_cmd("COMMIT TRANSACTION", dbh);
 }
 
-static int dblib_handle_rollback(pdo_dbh_t *dbh TSRMLS_DC)
+static int dblib_handle_rollback(pdo_dbh_t *dbh)
 {
-	return pdo_dblib_transaction_cmd("ROLLBACK TRANSACTION", dbh TSRMLS_CC);
+	return pdo_dblib_transaction_cmd("ROLLBACK TRANSACTION", dbh);
 }
 
-char *dblib_handle_last_id(pdo_dbh_t *dbh, const char *name, unsigned int *len TSRMLS_DC) 
+char *dblib_handle_last_id(pdo_dbh_t *dbh, const char *name, size_t *len)
 {
 	pdo_dblib_db_handle *H = (pdo_dblib_db_handle *)dbh->driver_data;
 
 	RETCODE ret;
 	char *id = NULL;
 
-	/* 
+	/*
 	 * Would use scope_identity() but it's not implemented on Sybase
 	 */
-	
+
 	if (FAIL == dbcmd(H->link, "SELECT @@IDENTITY")) {
 		return NULL;
 	}
-	
+
 	if (FAIL == dbsqlexec(H->link)) {
 		return NULL;
 	}
-	
+
 	ret = dbresults(H->link);
 	if (ret == FAIL || ret == NO_MORE_RESULTS) {
 		dbcancel(H->link);
@@ -226,7 +256,7 @@ char *dblib_handle_last_id(pdo_dbh_t *dbh, const char *name, unsigned int *len T
 	}
 
 	ret = dbnextrow(H->link);
-	
+
 	if (ret == FAIL || ret == NO_MORE_ROWS) {
 		dbcancel(H->link);
 		return NULL;
@@ -238,12 +268,29 @@ char *dblib_handle_last_id(pdo_dbh_t *dbh, const char *name, unsigned int *len T
 	}
 
 	id = emalloc(32);
-	*len = dbconvert(NULL, (dbcoltype(H->link, 1)) , (dbdata(H->link, 1)) , (dbdatlen(H->link, 1)), SQLCHAR, id, (DBINT)-1);
-		
+	*len = dbconvert(NULL, (dbcoltype(H->link, 1)) , (dbdata(H->link, 1)) , (dbdatlen(H->link, 1)), SQLCHAR, (BYTE *)id, (DBINT)-1);
+
 	dbcancel(H->link);
 	return id;
 }
 
+static int dblib_set_attr(pdo_dbh_t *dbh, zend_long attr, zval *val)
+{
+	switch(attr) {
+		case PDO_ATTR_TIMEOUT:
+			return 0;
+		default:
+			return 1;
+	}
+
+}
+
+static int dblib_get_attribute(pdo_dbh_t *dbh, zend_long attr, zval *return_value)
+{
+	/* dblib_handle *H = (pdo_pgsql_db_handle *)dbh->driver_data; */
+	return 0;
+}
+
 static struct pdo_dbh_methods dblib_methods = {
 	dblib_handle_closer,
 	dblib_handle_preparer,
@@ -252,22 +299,21 @@ static struct pdo_dbh_methods dblib_methods = {
 	dblib_handle_begin, /* begin */
 	dblib_handle_commit, /* commit */
 	dblib_handle_rollback, /* rollback */
-	NULL, /*set attr */
+	dblib_set_attr, /*set attr */
 	dblib_handle_last_id, /* last insert id */
 	dblib_fetch_error, /* fetch error */
-	NULL, /* get attr */
+	dblib_get_attribute, /* get attr */
 	NULL, /* check liveness */
 	NULL, /* get driver methods */
 	NULL, /* request shutdown */
 	NULL  /* in transaction */
 };
 
-static int pdo_dblib_handle_factory(pdo_dbh_t *dbh, zval *driver_options TSRMLS_DC)
+static int pdo_dblib_handle_factory(pdo_dbh_t *dbh, zval *driver_options)
 {
 	pdo_dblib_db_handle *H;
 	int i, nvars, nvers, ret = 0;
-	int *val;
-	
+
 	const pdo_dblib_keyval tdsver[] = {
 		 {"4.2",DBVERSION_42}
 		,{"4.6",DBVERSION_46}
@@ -283,11 +329,9 @@ static int pdo_dblib_handle_factory(pdo_dbh_t *dbh, zval *driver_options TSRMLS_
 #endif
 		,{"10.0",DBVERSION_100}
 		,{"auto",0} /* Only works with FreeTDS. Other drivers will bork */
-		
+
 	};
 	
-	nvers = sizeof(tdsver)/sizeof(tdsver[0]);
-	
 	struct pdo_data_src_parser vars[] = {
 		{ "charset",	NULL,	0 }
 		,{ "appname",	"PHP " PDO_DBLIB_FLAVOUR,	0 }
@@ -296,11 +340,18 @@ static int pdo_dblib_handle_factory(pdo_dbh_t *dbh, zval *driver_options TSRMLS_
 		,{ "secure",	NULL,	0 } /* DBSETLSECURE */
 		,{ "version",	NULL,	0 } /* DBSETLVERSION */
 	};
-	
+
 	nvars = sizeof(vars)/sizeof(vars[0]);
+	nvers = sizeof(tdsver)/sizeof(tdsver[0]);
 	
 	php_pdo_parse_data_source(dbh->data_source, dbh->data_source_len, vars, nvars);
 
+	if (driver_options) {
+		int timeout = pdo_attr_lval(driver_options, PDO_ATTR_TIMEOUT, 30);
+		dbsetlogintime(timeout); /* Connection/Login Timeout */
+		dbsettime(timeout); /* Statement Timeout */
+	}
+
 	H = pecalloc(1, sizeof(*H), dbh->is_persistent);
 	H->login = dblogin();
 	H->err.sqlstate = dbh->error_code;
@@ -309,23 +360,23 @@ static int pdo_dblib_handle_factory(pdo_dbh_t *dbh, zval *driver_options TSRMLS_
 		goto cleanup;
 	}
 
-	DBERRHANDLE(H->login, (EHANDLEFUNC) error_handler);
-	DBMSGHANDLE(H->login, (MHANDLEFUNC) msg_handler);
-	
+	DBERRHANDLE(H->login, (EHANDLEFUNC) pdo_dblib_error_handler);
+	DBMSGHANDLE(H->login, (MHANDLEFUNC) pdo_dblib_msg_handler);
+
 	if(vars[5].optval) {
 		for(i=0;i<nvers;i++) {
 			if(strcmp(vars[5].optval,tdsver[i].key) == 0) {
 				if(FAIL==dbsetlversion(H->login, tdsver[i].value)) {
-					pdo_raise_impl_error(dbh, NULL, "HY000", "PDO_DBLIB: Failed to set version specified in connection string." TSRMLS_CC);		
+					pdo_raise_impl_error(dbh, NULL, "HY000", "PDO_DBLIB: Failed to set version specified in connection string.");
 					goto cleanup;
 				}
 				break;
 			}
 		}
-		
+
 		if (i==nvers) {
 			printf("Invalid version '%s'\n", vars[5].optval);
-			pdo_raise_impl_error(dbh, NULL, "HY000", "PDO_DBLIB: Invalid version specified in connection string." TSRMLS_CC);		
+			pdo_raise_impl_error(dbh, NULL, "HY000", "PDO_DBLIB: Invalid version specified in connection string.");
 			goto cleanup; /* unknown version specified */
 		}
 	}
@@ -341,7 +392,7 @@ static int pdo_dblib_handle_factory(pdo_dbh_t *dbh, zval *driver_options TSRMLS_
 			goto cleanup;
 		}
 	}
-	
+
 #if !PHP_DBLIB_IS_MSSQL
 	if (vars[0].optval) {
 		DBSETLCHARSET(H->login, vars[0].optval);
@@ -399,7 +450,7 @@ cleanup:
 	dbh->driver_data = H;
 
 	if (!ret) {
-		zend_throw_exception_ex(php_pdo_get_exception(), DBLIB_G(err).dberr TSRMLS_CC,
+		zend_throw_exception_ex(php_pdo_get_exception(), DBLIB_G(err).dberr,
 			"SQLSTATE[%s] %s (severity %d)",
 			DBLIB_G(err).sqlstate,
 			DBLIB_G(err).dberrstr,
